"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createFamilyLayout = void 0;
const profile_info_1 = require("./profile-info");
const render_profile_svg_1 = require("./render-profile-svg");
const root_layout_1 = require("./root-layout");
const family_layout_1 = require("./family-layout");
const index_1 = require("./index");
const point_1 = require("./point");
const rectangle_1 = require("./rectangle");
const simple_family_tree_model_1 = require("simple-family-tree-model");
function addParentFamiliesFromProfile(tree, layout, profile, parentGenerationCount) {
    // The profile's parents can be found by following the childInFamilies links
    const parentFamilies = profile.childInFamilies.getLinks();
    parentFamilies.forEach((element) => {
        const parentFamily = tree.findFamily(new simple_family_tree_model_1.FamilyLink(element.itemLink));
        if (parentFamily !== undefined) {
            // The root profile is a parent in this family
            addProfilesFromFamily(tree, layout, parentFamily, parentGenerationCount - 1);
        }
    });
}
function addProfilesFromFamily(tree, layout, family, childGenerationCount) {
    if (!layout.families.has(family.familyId.itemLink)) {
        const familyId = family.familyId.itemLink;
        const nextPoint = layout.findFamilyPosition(tree, family.familyId);
        const familyLayout = new family_layout_1.FamilyLayout(familyId, new rectangle_1.Rectangle(nextPoint, index_1.defaultFamilySize));
        // console.log("Add family: {}", family.familyId.itemLink, nextPoint);
        const children = family.children.getLinks();
        children.forEach((element) => {
            if (!layout.profileAlreadyInLayout(new simple_family_tree_model_1.ProfileLink(element.itemLink))) {
                const child = tree.findProfile(new simple_family_tree_model_1.ProfileLink(element.itemLink));
                if (child !== undefined) {
                    const profileId = element.itemLink;
                    // console.log(
                    //   "  Family: {} add child:{}",
                    //   family.familyId.itemLink,
                    //   profileId
                    // );
                    if (!layout.profiles.has(profileId)) {
                        const childPoint = new point_1.Point(familyLayout.rect.topLeft.x -
                            20 +
                            familyLayout.children.length *
                                render_profile_svg_1.profileSvgStyles.profileBox.horizontalOffset, familyLayout.rect.topLeft.y +
                            render_profile_svg_1.profileSvgStyles.profileBox.verticalOffsetChild);
                        familyLayout.children.push(profileId);
                        const placement = new rectangle_1.Rectangle(childPoint, render_profile_svg_1.profileSvgStyles.profileBox.size);
                        layout.profiles.set(profileId, profileId);
                        familyLayout.profiles.set(profileId, new profile_info_1.ProfileInfo(placement, child));
                        // console.log("Addchild: ", profileId);
                        if (childGenerationCount > 0) {
                            addSpousesAndChildrenFromProfile(tree, layout, child, childGenerationCount - 1);
                        }
                    }
                }
                else {
                    console.log("  Family: {} child {} missing", family.familyId.itemLink, element.itemLink);
                }
            }
            else {
                familyLayout.children.push(element.itemLink);
            }
        });
        const spouses = family.parents.getLinks();
        spouses.forEach((element) => {
            if (!layout.profileAlreadyInLayout(new simple_family_tree_model_1.ProfileLink(element.itemLink))) {
                const spouse = tree.findProfile(new simple_family_tree_model_1.ProfileLink(element.itemLink));
                if (spouse !== undefined) {
                    const profileId = element.itemLink;
                    // console.log(
                    //   "  Family: {} add parent:{}",
                    //   family.familyId.itemLink,
                    //   profileId
                    // );
                    if (!layout.profiles.has(profileId)) {
                        const spousePoint = new point_1.Point(familyLayout.rect.topLeft.x -
                            20 +
                            familyLayout.parents.length *
                                render_profile_svg_1.profileSvgStyles.profileBox.horizontalOffset, familyLayout.rect.topLeft.y +
                            render_profile_svg_1.profileSvgStyles.profileBox.verticalOffsetParent);
                        const placement = new rectangle_1.Rectangle(spousePoint, render_profile_svg_1.profileSvgStyles.profileBox.size);
                        layout.profiles.set(profileId, profileId);
                        familyLayout.profiles.set(profileId, new profile_info_1.ProfileInfo(placement, spouse));
                        familyLayout.parents.push(profileId);
                        // console.log("Addchild: ", profileId);
                        if (childGenerationCount > 0) {
                            addSpousesAndChildrenFromProfile(tree, layout, spouse, childGenerationCount - 1);
                        }
                    }
                }
                else {
                    console.log("  Family: {} parent {} missing", family.familyId.itemLink, element.itemLink);
                }
            }
            else {
                familyLayout.parents.push(element.itemLink);
            }
        });
        layout.families.set(family.familyId.itemLink, familyLayout);
    }
}
function addSpousesAndChildrenFromProfile(tree, layout, profile, childGenerationCount) {
    // The profile's children and spouses can be found by following the parentInFamilies links
    const childFamilies = profile.parentInFamilies.getLinks();
    childFamilies.forEach((element) => {
        const family = tree.findFamily(new simple_family_tree_model_1.FamilyLink(element.itemLink));
        if (family !== undefined) {
            // The root profile is a child in this family
            addProfilesFromFamily(tree, layout, family, childGenerationCount - 1);
        }
    });
}
function createFamilyLayout(tree, startProfile, parentCount, childCount) {
    const layout = new root_layout_1.RootLayout();
    const profile = tree.findProfile(startProfile);
    if (profile !== undefined) {
        // We start by setting the root profile
        const startProfileId = profile.profileId.itemLink;
        layout.anchorProfile = startProfileId;
        // console.log(
        //   "show profile " +
        //     startProfileId +
        //     " child in " +
        //     profile.childInFamilies.getLinks().length +
        //     " families, parent in " +
        //     profile.parentInFamilies.getLinks().length +
        //     " families "
        // );
        // Now we check if the profile is a child in any families
        if (childCount > 0) {
            addSpousesAndChildrenFromProfile(tree, layout, profile, childCount - 1);
        }
        // Now we check if the profile is a parent in any families
        if (parentCount > 0) {
            addParentFamiliesFromProfile(tree, layout, profile, parentCount - 1);
        }
        const partners = (0, simple_family_tree_model_1.getPartners)(tree, profile.profileId);
        if (partners.length > 0) {
            partners.forEach((partnerLink) => {
                const partner2 = tree.findProfile(partnerLink);
                if (partner2 != undefined) {
                    addParentFamiliesFromProfile(tree, layout, partner2, parentCount - 1);
                }
            });
        }
    }
    return layout;
}
exports.createFamilyLayout = createFamilyLayout;
